@@ -11,13 +11,13 @@ test_that("income tax checks", {
   expect_equal(suppressWarnings(income_tax(30e3)), income_tax(30e3, date2fy(Sys.Date())))
   
   # not fy
-  expect_error(income_tax(1, "2015-17"), regexp = "not in correct form")
+  expect_error(income_tax(1, "2015-17"), regexp = "financial year")
   
   # not implemented
   expect_error(income_tax(1, "2030-31", allow.forecasts = TRUE))
   
   # NA
-  expect_warning(income_tax(-1, "2014-15"), regexp = "Negative")
+  expect_warning(income_tax(-1, "2014-15"), regexp = "Negative", ignore.case = TRUE)
   
   # not a data frame
   expect_error(income_tax(1, "2013-14", .dots.ATO = "foo"))
@@ -44,19 +44,33 @@ test_that("income_tax returns known results",{
   
   expect_equal(income_tax(30e3, fy.year = "2013-14"), 2247)
   expect_equal(income_tax(40e3, fy.year = "2013-14"), 4747)
-  expect_equal(income_tax(40e3, "2013-14", family_status = "family", n_dependants = 1L), 4394.70)
+  # expect_equal(income_tax(40e3, "2013-14", family_status = "family", n_dependants = 1L), 4394.70)
+  expect_equal(income_tax(40e3, "2013-14", 
+                          .dots.ATO = data.table(sp_flag = TRUE,
+                                                 c_depend_child = 1L,
+                                                 spc_rebate_income = 0L,
+                                                 c_age_30_june = 42L)),
+               4394.70)
   # different rounding treatment.
-  expect_equal(round(income_tax(40e3,
-                                "2013-14",
-                                family_status = "family",
-                                n_dependants = 0L,
-                                age = 66)),
-               2882)
+  # Test dropped
+  # expect_equal(round(income_tax(40e3,
+  #                               "2013-14",
+  #                               family_status = "family",
+  #                               n_dependants = 0L,
+  #                               age = 66)),
+  #              2882)
+  # expect_equal(income_tax(40e3, 
+  #                         "2013-14",
+  #                         family_status = "family",
+  #                         n_dependants = 2L,
+  #                         .dots.ATO = data.frame(Spouse_adjusted_taxable_inc = 30e3)),
+  #              4747)
   expect_equal(income_tax(40e3, 
-                          "2013-14",
-                          family_status = "family",
-                          n_dependants = 2L,
-                          .dots.ATO = data.frame(Spouse_adjusted_taxable_inc = 30e3)),
+                          "2013-14", 
+                          .dots.ATO = data.table(sp_flag = TRUE,
+                                                 c_depend_child = 2L,
+                                                 spc_rebate_income = 30e3L,
+                                                 c_age_30_june = 42L)),
                4747)
   
   expect_equal(income_tax(31993, fy.year = "2014-15"), 2815.53)
@@ -66,11 +80,11 @@ test_that("income_tax returns known results",{
   expect_equal(income_tax(26010, fy.year = "2014-15"), 1550.30, tol = 1, scale = 1)
   
   # Test rolling now no longer the default
-  expect_equal(rolling_income_tax(31993, fy.year = "2014-15"), 2815.53)
-  expect_equal(rolling_income_tax(31993, fy.year = "2014-15", age = 70), 0)
-  expect_equal(rolling_income_tax(135288, fy.year = "2014-15"), 40709, tol = 1, scale = 1)
-  expect_equal(rolling_income_tax(41636, fy.year = "2014-15"), 5535.95, tol = 1, scale = 1)
-  expect_equal(rolling_income_tax(26010, fy.year = "2014-15"), 1550.30, tol = 1, scale = 1)
+  # expect_equal(rolling_income_tax(31993, fy.year = "2014-15"), 2815.53)
+  # expect_equal(rolling_income_tax(31993, fy.year = "2014-15", age = 70), 0)
+  # expect_equal(rolling_income_tax(135288, fy.year = "2014-15"), 40709, tol = 1, scale = 1)
+  # expect_equal(rolling_income_tax(41636, fy.year = "2014-15"), 5535.95, tol = 1, scale = 1)
+  # expect_equal(rolling_income_tax(26010, fy.year = "2014-15"), 1550.30, tol = 1, scale = 1)
   
   
 })
